#################################################################################
# Created       : Martin Foster
# Modified      : 27/12/2002
#################################################################################
#
# Image - Image handling module for Ethereal-Realms 
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Image;


# Image manipulation
use Image::Magick;

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;				# Class
        my $database = shift;				# Database handle
	my $basedir  = shift;				# Base directory

	my $self;					# Reference to self

	my %system;					# System hash


	#####################
	# Program files

        # Link with hash
        $database->GetHashSystem(\%system);


	# Base directory
	$basedir = (defined($basedir)) ? $basedir : $system{'SetBaseDir'};


	# Generate hash
        %{$self}  = (
               BASEDIR => $basedir,			# Default directory
               LEGACY  => $system{'SetLegacy'},		# Legacy lib support
	       PIXEL   => $system{'SetThumbnail'},      # Thumbnail demensions
	 );


	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


####################
# Set
#
# Sets the size of the image to specific demensions as determined
# by the parameters received.

sub Set
{
	####################
	# Data members

	my $self    = shift;				# Self referencing link
	my $image   = shift;				# Specific image
	my $max     = shift;				# Max demensions			
	my $magick  = shift;				# Image handle

	my $basedir = $self->{'BASEDIR'};		# Base working directory
	my $pixel   = $self->{'PIXEL'};			# Pixel demensions

	my $height;					# Image height
	my $width;					# Image width


	####################
	# Program area

	# Create instance
	$magick = new Image::Magick;

	# Open
	$magick->Read("$basedir/$image");

	
	# Pull dimensions
	($width, $height) = $self->Size($image, $magick);

	# Useable
	if ((defined($width))
	 && (defined($height)))
	{
		# Resize calculations
		# Height
		if ($height > $width)
		{
			# Calculate demensions
			$width  = int(($width * $max) / $height);
			$height = $max;
		}

		# Width
		else
		{
			# Calculate demensions
			$height = int(($height * $max) / $width);
			$width  = $max;
		}


		# Image manipulation
		# Resize operation
		$magick->Resize(width=>$width, height=>$height);

		# Write
		$self->Write("$image", $magick);
	}
}


####################
# Size
#
# A simple file size handling subroutine that is used to gain various
# information on images.  Needed for basic chatting, thumbnail creation
# and image resizing.
#
# Due to possible problems related to the use of the Image::Magick
# library, there is an option for legacy support which will allow one
# to use Image::Size instead.

sub Size
{
	####################
	# Data members

	my $self    = shift;				# Self referencing link
	my $image   = shift;				# Specific image
	my $magick  = shift;				# Image handle

	my $basedir = $self->{'BASEDIR'};		# Base working directory
	my $legacy  = $self->{'LEGACY'};		# Legacy image handling

	my $format;					# Image format
	my $height;					# Image height
	my $size;					# Image size
	my $width;					# Image width


	####################
	# Program area

	# Modern Image::Magick Operation
	if ($legacy < 1)
	{
		# Create instance
		$magick = new Image::Magick unless (defined($magick));

		# Retreive parameters
		($width, $height, $size, $format) = $magick->Ping("$basedir/$image");
		($width, $height, $size, $format) = split(/,/, $width) if ($width =~ /,/);

		# File size in Kilobytes
		$size = int($size/1024);

		# Return values
		return $width, $height, $size, $format;
	}

	# Legacy Image::Size Support
	else
	{
		# Load module
		use Image::Size;

		# Width and height
		($width, $height) = imgsize("$basedir/$image");

		# File size in Kilobytes
		$size = -s "$basedir/$image";
		$size = int($size/1024);

		# Return values
		return $width, $height, $size, $format;
	}
}


####################
# Thumbnail gen
#
# Used to generate thumbnails of images received.  This is accomplished by
# generating three seperate images, the user selects the best likeness
# using alternate means.

sub Thumbnail
{
	####################
	# Data members

	my $self    = shift;				# Self referencing link
	my $image   = shift;				# Specific image
	my $magick  = shift;				# Image handle

	my $basedir = $self->{'BASEDIR'};		# Base working directory
	my $pixel   = $self->{'PIXEL'};			# Pixel demensions

	my $format;					# Image format
	my $height;					# Image height
	my $size;					# Image size in Kilobytes
	my $width;					# Image width

	my $x;						# X coordinates
	my $y;						# Y coordinates

	my $mid;					# Mid way cut
	my $neg;					# Negative cut
	my $pos;					# Positive cut


	####################
	# Program area

	# Create instance
	$magick = new Image::Magick;

	# Open
	$magick->Read("$basedir/$image");

	
	# Pull dimensions
	($width, $height, $size, $format) = $self->Size($image, $magick);


	# Resize calculations
	# Height
	if ($height < $width)
	{
		# Calculate demensions
		$width  = int(($width * $pixel) / $height);
		$height = $pixel;

		# Crop co-ords
		$x = int(($width - $pixel) / 2);
		$y = 0;
	}

	# Width
	else
	{
		# Calculate demensions
		$height = int(($height * $pixel) / $width);
		$width  = $pixel;

		# Crop co-ords
		$x = 0;
		$y = int(($height - $pixel) / 2);
	}

	# Image manipulation
	# Resize operation
	$magick->Resize(width=>$width, height=>$height);

	# Safety set
	$magick->Set(magick=>'JPEG');

	# Filename adapt
	$image =~ /(\w+)\.\w+$/;
	$image = $1;


	# Crop
	# Negative
	$neg = $magick->Clone();
	$neg->Crop(width=>$pixel, height=>$pixel, x=>0, y=>0);

	# Write
	$self->Write("thumb-$image-neg.jpg", $neg);


	# Middle
	$mid = $magick->Clone();
	$mid->Crop(width=>$pixel, height=>$pixel, x=>$x, y=>$y);

	# Write
	$self->Write("thumb-$image-mid.jpg", $mid);


	# Double
	$x += $x;
	$y += $y;

	# Positive
	$pos = $magick->Clone();
	$pos->Crop(width=>$pixel, height=>$pixel, x=>$x, y=>$y);

	# Write
	$self->Write("thumb-$image-pos.jpg", $pos);
}


####################
# Write
#
# Image writer, used to write the final changes of images when
# the time comes.

sub Write
{
	####################
	# Data members

	my $self    = shift;				# Self referencing link
	my $image   = shift;				# Specific image
	my $magick  = shift;				# Image handle

	my $basedir = $self->{'BASEDIR'};		# Base working directory


	####################
	# Program Area

 	# Rewrite thumbnail
	# Open filehandle
	if(open(IMAGE, ">$basedir/$image"))
	{
		# Write
		$magick->Write(file=>\*IMAGE, filename=>$image);	

		# Close filehandle
		close(IMAGE);

		# Return
		return 0;
	}
	else
	{
		# Return error message
		return $!;
	}
}

1;								# Return true
